package ui;
/*
 * This file is part of FilamentSensor - A tool for filament tracking from cell images
 *
 * Copyright (C) 2011-2013 Julian Rüger
 *
 * FilamentSensor is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA
 * or see <http://www.gnu.org/licenses/>.
 */

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyEvent;

import javax.swing.Icon;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;
import javax.swing.KeyStroke;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

import core.DataFilaments;
import core.DataImage;

class MainWindowMenu extends JMenuBar implements ChangeListener
{
    // Single instance
    private static MainWindowMenu the;
    
    // Top level entries
    private JMenu file, edit, view, filament, help;
    // Menu items
    static JMenuItem f_new, f_open, f_csv, f_update, f_save, f_saveas, f_print, f_close, f_exit,
                     e_undo, e_redo, e_settings,
                     v_org, v_double, v_zoomin, v_zoomout,
                     fl_delete,
                     h_documentation;

    private ActionListener m_listener;

    private MainWindowMenu(ActionListener listener)
    {
        super();
        this.m_listener = listener;

        file = new JMenu("File");
        file.setMnemonic(KeyEvent.VK_F);
        this.add(file);

        edit = new JMenu("Edit");
        file.setMnemonic(KeyEvent.VK_E);
        this.add(edit);

        view = new JMenu("View");
        file.setMnemonic(KeyEvent.VK_V);
        this.add(view);

        filament = new JMenu("Filament");
        file.setMnemonic(KeyEvent.VK_I);
        this.add(filament);

        help = new JMenu("Help");
        file.setMnemonic(KeyEvent.VK_H);
        this.add(help);

        ////////////////////////////// File menu //////////////////////////////

        f_new = menuItem("New images...", null, KeyEvent.VK_N, ActionEvent.CTRL_MASK,
                         "Start a new project from a folder with images.");
        file.add(f_new);
        f_open = menuItem("Open project...", null, KeyEvent.VK_O, ActionEvent.CTRL_MASK,
                          "Open an existing project XML file.");
        file.add(f_open);
        f_csv = menuItem("Import csv...", null, KeyEvent.VK_C, ActionEvent.CTRL_MASK,
                         "Import image data from csv file.");
        f_csv.setEnabled(false);
        file.add(f_csv);
        f_update = menuItem("Update old project...", null, KeyEvent.VK_U, ActionEvent.CTRL_MASK,
                            "Update global cell data for an existing project.");
        file.add(f_update);

        file.addSeparator();

        f_save = menuItem("Save project...", null, KeyEvent.VK_S, ActionEvent.CTRL_MASK,
                          "Save the changes you made in the current project.");
        f_save.setEnabled(false);
        file.add(f_save);
        f_saveas = menuItem("Save project as...", null, KeyEvent.VK_S, ActionEvent.CTRL_MASK | ActionEvent.SHIFT_MASK,
                            "Save the current project to a new file");
        file.add(f_saveas);

        file.addSeparator();

        f_print = menuItem("Print...", null, KeyEvent.VK_P, ActionEvent.CTRL_MASK, "Print the current filaments");
        f_print.setEnabled(false);
        file.add(f_print);

        file.addSeparator();

        f_close = menuItem("Close", null, KeyEvent.VK_W, ActionEvent.CTRL_MASK, "Close the current tab");
        file.add(f_close);
        f_exit = menuItem("Exit", null, KeyEvent.VK_X, ActionEvent.CTRL_MASK, "Exit FilPicker");
        file.add(f_exit);

        // //////////////////////////// Edit menu //////////////////////////////

        e_undo = menuItem("Undo", null, KeyEvent.VK_Z, ActionEvent.CTRL_MASK, "Undo the last operation");
        e_undo.setEnabled(false);
        edit.add(e_undo);

        e_redo = menuItem("Redo", null, KeyEvent.VK_Y, ActionEvent.CTRL_MASK, "Redo the last undone operation");
        e_redo.setEnabled(false);
        edit.add(e_redo);

        edit.addSeparator();

        // TODO: copy, cut paste...

        edit.addSeparator();

        e_settings = menuItem("Settings...", null, 0, 0, "Open the FilPicker settings dialog");
        edit.add(e_settings);

        // //////////////////////////// View menu //////////////////////////////

        v_zoomin = menuItem("Zoom In", IconCollection.ZOOMIN16, KeyEvent.VK_PLUS, ActionEvent.CTRL_MASK,
                            "Increase the zoom factor");
        view.add(v_zoomin);

        v_zoomout = menuItem("Zoom Out", IconCollection.ZOOMOUT16, KeyEvent.VK_MINUS, ActionEvent.CTRL_MASK,
                             "Decrease the zoom factor");
        view.add(v_zoomout);

        v_org = menuItem("Zoom original (1:1)", IconCollection.ZOOMORG16, KeyEvent.VK_0, ActionEvent.CTRL_MASK,
                         "Set zoom factor to 100%");
        view.add(v_org);

        ////////////////////////////// Filament menu ////////////////////////////////

        fl_delete = menuItem("Delete", null, KeyEvent.VK_DELETE, 0, "Delete the currently selected filament(s)");
        fl_delete.setEnabled(false);
        filament.add(fl_delete);

        ////////////////////////////// Help menu ////////////////////////////////

        h_documentation = menuItem("Information", null, KeyEvent.VK_H, ActionEvent.CTRL_MASK,
                                   "Show usage information.");
        help.add(h_documentation);
    }

    static void create (ActionListener alistener)
    {
        if (the == null)
        {
            the = new MainWindowMenu(alistener);
        }
    }

    static MainWindowMenu get()
    {
        return the;
    }

    private JMenuItem menuItem(String label, Icon icon, int acc_key, int acc_mask, String description)
    {
        JMenuItem item = new JMenuItem(label);

        if (icon != null)
        {
            item.setIcon(icon);
        }
        if (acc_key != 0)
        {
            item.setAccelerator(KeyStroke.getKeyStroke(acc_key, acc_mask));
        }
        if (description != null && description != "")
        {
            item.getAccessibleContext().setAccessibleDescription(description);
        }

        item.addActionListener(m_listener);
        return item;
    }

    @Override
    public void stateChanged(ChangeEvent e)
    {
        if (e == null)
        {
            return;
        }

        Object src = e.getSource();
        if (src instanceof DataImage)
        {
            DataImage data = (DataImage) src;
            f_save.setEnabled(data.hasChanged());
        }
        else if (src instanceof DataFilaments)
        {
            DataFilaments filament_container = (DataFilaments) src;
            e_undo.setEnabled(filament_container.canUndoChain());
            e_redo.setEnabled(filament_container.canRedoChain());
            fl_delete.setEnabled(!filament_container.getSelection().isEmpty());
        }
    }

    @Override
    public Object clone() throws CloneNotSupportedException
    {
        throw new CloneNotSupportedException();
    }

}
