package ui;
/*
 * This file is part of FilamentSensor - A tool for filament tracking from cell images
 *
 * Copyright (C) 2011-2013 Julian Rüger
 *               2014      Benjamin Eltzner
 *
 * FilamentSensor is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA
 * or see <http://www.gnu.org/licenses/>.
 */


import java.awt.Dimension;
import java.awt.Point;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JPanel;

import core.DataImage;
import core.DataParameters;
import core.Misc;

class PopUpWidthMap extends JFrame implements ActionListener, Misc
{
    private JPanel m_panel_content, m_panel_controls;
    private PanelImage m_panel_image;
    private SlideSpinPane m_slider_tolerance;
    private JButton m_button_close;

    private DataImage m_data;

    public PopUpWidthMap(DataImage data)
    {
        super("Width test");
        setDefaultCloseOperation(DISPOSE_ON_CLOSE);

        this.m_data = data;

        m_panel_content = new JPanel();
        m_panel_content.setLayout(new BoxLayout(m_panel_content, BoxLayout.Y_AXIS));

        m_panel_image = new PanelImage();
        m_panel_content.add(m_panel_image);

        m_panel_controls = new JPanel();
        m_panel_controls.setLayout(new BoxLayout(m_panel_controls, BoxLayout.X_AXIS));

        m_panel_controls.add(Box.createHorizontalGlue());

        m_slider_tolerance = new SlideSpinPane("Tolerance (%):", data.getParameter(DataParameters.Trace.tolerance), 0, 50, 1);
        m_slider_tolerance.setMinimumSize(new Dimension(250, 70));
        m_slider_tolerance.setTickSpacing(1, 10);
        m_slider_tolerance.addActionListener(this, "");
        m_panel_controls.add(m_slider_tolerance);

        m_button_close = new JButton("  Close  ");
        m_button_close.addActionListener(this);
        m_panel_controls.add(m_button_close);

        m_panel_controls.add(Box.createHorizontalGlue());

        m_panel_content.add(m_panel_controls);
        calcWidthMap();

        setDefaultLookAndFeelDecorated(false);
        setContentPane(m_panel_content);
        // setDefaultCloseOperation(WindowConstants.DO_NOTHING_ON_CLOSE);
        // addWindowListener(this);

        setMinimumSize(new Dimension(MainWindow.MIN_WIDTH, MainWindow.MIN_HEIGHT));
        Dimension size = MainWindow.frame().getSize();
        size.width -= 100;
        size.height -= 100;
        this.setSize(size);
        Point location = MainWindow.frame().getLocation();
        location.x += 50;
        location.y += 50;
        this.setLocation(location);
        setVisible(true);
    }

    @Override
    public void actionPerformed(ActionEvent e)
    {
        Object src = e.getSource();

        if (src == m_button_close)
        {
            dispose();
        }
        else if (src == m_slider_tolerance)
        {
            m_data.setParameter(DataParameters.Trace.tolerance, m_slider_tolerance.getValue());
            calcWidthMap();
        }
    }

    private void calcWidthMap()
    {
        Pair<Int2D, Integer> pair = m_data.calculateWidthMap(m_slider_tolerance.getValue() / 100.0);
        m_panel_image.setImage(ImageFactory.makeWidthMap(pair), false);
    }

    // TODO: markieren gimp html notation a2c4e5, 162, 196, 229

}
