package ui;
/*
 * This file is part of FilamentSensor - A tool for filament tracking from cell images
 *
 * Copyright (C) 2011-2013 Julian Rüger
 *               2013      Patricia Burger, Benjamin Eltzner
 *
 * FilamentSensor is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA
 * or see <http://www.gnu.org/licenses/>.
 */

import javax.swing.JOptionPane;
import javax.swing.JTabbedPane;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

import core.DataImage;
import core.DataParameters;

import java.awt.Component;
import java.io.File;

public class TabBar extends JTabbedPane implements ChangeListener
{
    private ChangeListener menu;
    public static int tab_divider_location;

    public TabBar(ChangeListener menu)
    {
        super();

        addChangeListener(this);
        this.menu = menu;

        setTabLayoutPolicy(JTabbedPane.SCROLL_TAB_LAYOUT);
        // UIManager.getDefaults().put("TabbedPane.TabAreaInsets", new
        // Insets(0,0,0,0));
        // UIManager.getDefaults().put("TabbedPane.tabsOverlapBorder", true);
    }

    public boolean setThumbnailTab(boolean load)
    {

        //boolean newtab = true;
        ThumbnailView tab = null;

        if (containsThumbnailView())
        {
            if (FilamentSensor.DEBUG)
            {
                System.out.println("There is already a ThumbnailView");
            }

            //newtab = false;

            switch (JOptionPane.showOptionDialog(MainWindow.frame(), "If you open a new"
                    + " project, the current project will be closed. Save current project?", "Save current project?",
                    JOptionPane.YES_NO_CANCEL_OPTION, JOptionPane.WARNING_MESSAGE, null, null, null))
            {
            case JOptionPane.YES_OPTION:
                if (!saveProject()
                        && JOptionPane.showOptionDialog(MainWindow.frame(), "Saving failed! Open"
                                + " new project anyway?", "Discard unsaved changes?", JOptionPane.YES_NO_OPTION,
                                JOptionPane.ERROR_MESSAGE, null, null, null) != JOptionPane.YES_OPTION)
                {
                    return false;
                }
                // else: fall through.
            case JOptionPane.NO_OPTION:
                break;
            default:
                return false;
            }
        }

        IOProjectData data = new IOProjectData(load);
        if (!data.valid())
        {
            return false;
        }
        String title = data.getName();
        String unique = makeUnique(title);

        closeAll(true);
        tab = new ThumbnailView(this, data);

        this.addTab(unique, null, null, title);

        int i = this.indexOfTab(unique);
        setTabComponentAt(i, tab);
        setComponentAt(i, tab.scroller);
        setToolTipTextAt(i, title);
        setSelectedIndex(i);

        if (FilamentSensor.DEBUG)
        {
            System.out.println("New thumbnail tab created");
        }

        return true;
    }

    public boolean newPickingView(File image_file, File data_file, String image_path,
                                  DataParameters default_params, boolean force_defaults)
    {
        DataImage data = null;
        String name = "";
        String path = "";

        if (data_file == null)
        {
            data = new DataImage(image_file, default_params);
            name = image_file.getName();
            path = image_file.getPath();
        }
        else
        {
            IOImageData xml_file = new IOImageData(data_file);

            if (FilamentSensor.DEBUG)
            {
                System.out.println("New file from " + data_file.getName());
            }

            name = xml_file.getName();
            path = xml_file.getPath();

            // Did something go wrong?
            if (name == null)
            {
                return false;
            }

            data = xml_file.read(image_path, default_params, force_defaults);

            if (FilamentSensor.DEBUG)
            {
                System.out.println("New DataImage created from" + data_file);
            }

            if (data == null)
            {
                // something went wrong
                if (FilamentSensor.DEBUG)
                {
                    System.err.println("FilamentSensor TabBar.newPickingView: " +
                    		           "Failed to parse file '" + name + "'.\nfile: " + xml_file +
                    		           "\ndata: " + data);
                }
                return false;
            }
        }

        // Add tab
        String title = PickingView.stripExtension(name);
        String unique = makeUnique(title);

        PickingView tab = new PickingView(this, unique);
        tab.setData(data);

        this.addTab(unique, null, null, title);
        int i = this.indexOfTab(unique);
        setTabComponentAt(i, tab);
        setComponentAt(i, tab.m_picking_main_widget);
        tab.showImage(); // TODO: Improve.

        setToolTipTextAt(i, path);
        setSelectedIndex(i);
        MainWindowMenu.f_csv.setEnabled(true);

        if (FilamentSensor.DEBUG)
        {
            System.out.println("New picking tab created:\n\tTitle: \"" + unique +
                               "\"\n\tIndex: " + i + ", " + indexOfTabComponent(tab));
        }

        return true;
    }

    public String setTitle(AbstractTab tab, String title)
    {
        int i = indexOfTabComponent(tab);

        if (i < 0)
        {
            System.err.println("FilamentSensor TabBar.setTitle: Tab not registered in tab bar!");
            return title;
        }
        if (title == getTitleAt(i))
        {
            return title;
        }

        String unique = makeUnique(title);
        setTitleAt(i, unique);
        return unique;
    }

    public void setTooltip(AbstractTab tab, String tooltip)
    {
        setToolTipTextAt(indexOfTabComponent(tab), tooltip);
    }

    private AbstractTab tabAt(int index)
    {
        if (index < 0)
        {
            if (FilamentSensor.DEBUG)
            {
                System.err.println("FilamentSensor TabBar.tabAt: Tab index is " + index);
            }
            return null;
        }

        Component tab = getTabComponentAt(index);

        if (tab == null)
        {
            if (FilamentSensor.DEBUG)
            {
                System.err.println("FilamentSensor TabBar.tabAt: Tab at index " + index + " is null.");
            }
            return null;
        }

        if (tab instanceof PickingView)
        {
            return (PickingView) tab;
        }

        if (tab instanceof ThumbnailView)
        {
            return (ThumbnailView) tab;
        }

        return null;
    }

    // TODO: Return AbstractTab and clean up the fallout.
    public PickingView getCurrentTab()
    {
        if (!(tabAt(getSelectedIndex()) instanceof PickingView))
        {
            return null;
        }

        return (PickingView) (tabAt(getSelectedIndex()));
    }

    private boolean containsThumbnailView()
    {
        for (int i = 0; i < getTabCount(); i++)
        {
            Component tab = getTabComponentAt(i);
            if (tab instanceof ThumbnailView && tab != null)
            {
                if (FilamentSensor.DEBUG)
                {
                    System.out.println("ThumbnailView found at index: " + i);
                }

                return true;
            }
        }
        return false;
    }

    public boolean importCSV()
    {
        PickingView tab = getCurrentTab();

        if (tab == null)
        {
            return false;
        }
        return tab.importCSV();
    }

    public boolean saveProject()
    {
        for (int i = 0; i < getTabCount(); i++)
        {
            Component tab = getTabComponentAt(i);
            if (tab instanceof ThumbnailView && tab != null)
            {
                return ((ThumbnailView) tab).saveData();
            }
        }
        return false;
    }

    public ThumbnailView getThumbnailView()
    {
        for (int i = 0; i < getTabCount(); i++)
        {
            Component tab = getTabComponentAt(i);
            if (tab instanceof ThumbnailView && tab != null)
            {
                return (ThumbnailView) tab;
            }
        }
        return null;
    }

    private String makeUnique(String title)
    {
        int i = 0;
        String unique = title;
        while (this.indexOfTab(unique) != -1)
        {
            unique = title + " (" + (++i) + ")";
        }
        return unique;
    }

    public boolean closeTab(int index, boolean quiet)
    {
        if (FilamentSensor.DEBUG)
        {
            System.out.println("Trying to close tab with index: " + index);
        }

        AbstractTab tab = tabAt(index);

        if (tab == null)
        {
            System.err.println("FilamentSensor TabBar.closeTab: The tab is null.");
        }

        if (!quiet && !tab.readyToClose())
        {
            return false;
        }

        if (indexOfTabComponent(tabAt(index)) != index)
        {
            System.err.println("FilamentSensor TabBar.closeTab: tab-index mismatch!\nlist: " + index +
                               "\npane: " + indexOfTabComponent(tabAt(index)) + "\n");
            return false;
        }

        this.remove(index);
        //stateChanged(null);
        return true;
    }

    public boolean closeCurrentTab(boolean quiet)
    {
        return closeTab(getSelectedIndex(), quiet);
    }

    public boolean closeAll(boolean quiet)
    {
        while (getTabCount() > 0)
        {
            if (!closeTab(getSelectedIndex(), quiet))
            {
                return false;
            }
        }
        return true;
    }

    public void updateMenu(ChangeEvent e)
    {
        menu.stateChanged(e);
    }

    @Override
    public void stateChanged(ChangeEvent e)
    {
        AbstractTab tab = tabAt(getSelectedIndex());

        if (tab == null)
        {
            return;
        }

        if (tab instanceof PickingView)
        {
            // update the SplitPane-divider location
            ((PickingView) tab).m_picking_main_widget.setDividerLocation(tab_divider_location);
        }
    }
}
